@@ -1,26 +1,14 @@
 /* vi: ts=8 sts=4 sw=4
- *
- * kiconloader.cpp: An icon loader for KDE with theming functionality.
- *
- * This file is part of the KDE project, module kdeui.
- * Copyright (C) 2000 Geert Jansen <jansen@kde.org>
- *                    Antonio Larrosa <larrosa@kde.org>
- *               2010 Michael Pyne <mpyne@kde.org>
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License version 2 as published by the Free Software Foundation.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public License
- * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
- */
+
+    kiconloader.cpp: An icon loader for KDE with theming functionality.
+
+    This file is part of the KDE project, module kdeui.
+    SPDX-FileCopyrightText: 2000 Geert Jansen <jansen@kde.org>
+    SPDX-FileCopyrightText: 2000 Antonio Larrosa <larrosa@kde.org>
+    SPDX-FileCopyrightText: 2010 Michael Pyne <mpyne@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.0-only
+*/
 
 #include "kiconloader.h"
 
@@ -44,9 +32,9 @@
 #include <QGuiApplication>
 
 // kdecore
-#include <kconfiggroup.h>
+#include <KConfigGroup>
 #include <kshareddatacache.h>
-#include <ksharedconfig.h>
+#include <KSharedConfig>
 #ifdef QT_DBUS_LIB
 #include <QDBusConnection>
 #include <QDBusMessage>
@@ -61,10 +49,10 @@
 #include "debug.h"
 
 //kwidgetsaddons
-#include <kpixmapsequence.h>
+#include <KPixmapSequence>
 
-#include <kcolorscheme.h>
-#include <kcompressiondevice.h>
+#include <KColorScheme>
+#include <KCompressionDevice>
 
 namespace {
 // Used to make cache keys for icons with no group. Result type is QString*
@@ -560,9 +548,8 @@ void KIconLoaderPrivate::_k_refreshIcons(int group)
 {
     KSharedConfig::Ptr sharedConfig = KSharedConfig::openConfig();
     sharedConfig->reparseConfiguration();
-
     const QString newThemeName = sharedConfig->group("Icons")
-                                               .readEntry("Theme", QString());
+                                               .readEntry("Theme", QStringLiteral("breeze"));
     if (!newThemeName.isEmpty()) {
         // If we're refreshing icons the Qt platform plugin has probably
         // already cached the old theme, which will accidentally filter back
@@ -1135,6 +1122,23 @@ QString KIconLoaderPrivate::findMatchingIcon(const QString &name, int size, qrea
             }
         }
     }
+
+    if (path.isEmpty()) {
+        const QStringList fallbackPaths = QIcon::fallbackSearchPaths();
+
+        for (const QString &path : fallbackPaths) {
+            const QString extensions[] = { QStringLiteral(".png"), QStringLiteral(".svg"), QStringLiteral(".svgz"), QStringLiteral(".xpm") };
+
+            for (const QString &ext : extensions) {
+                const QString file = path + '/' + name + ext;
+
+                if (QFileInfo::exists(file)) {
+                    return file;
+                }
+            }
+        }
+    }
+
     return path;
 }
 
@@ -1680,8 +1684,7 @@ bool KIconLoader::alphaBlending(KIconLoader::Group group) const
     return true;
 }
 
-// deprecated
-#ifndef KICONTHEMES_NO_DEPRECATED
+#if KICONTHEMES_BUILD_DEPRECATED_SINCE(5, 0)
 QIcon KIconLoader::loadIconSet(const QString &name, KIconLoader::Group g, int s,
                                bool canReturnNull)
 {
@@ -1699,53 +1702,63 @@ QIcon KIconLoader::loadIconSet(const QString &name, KIconLoader::Group g, int s,
 
 // Easy access functions
 
-#ifndef KICONTHEMES_NO_DEPRECATED
+#if KICONTHEMES_BUILD_DEPRECATED_SINCE(5, 63)
 QPixmap DesktopIcon(const QString &name, int force_size, int state, const QStringList &overlays)
 {
     KIconLoader *loader = KIconLoader::global();
     return loader->loadIcon(name, KIconLoader::Desktop, force_size, state, overlays);
 }
+#endif
 
-// deprecated
+#if KICONTHEMES_BUILD_DEPRECATED_SINCE(5, 0)
 QIcon DesktopIconSet(const QString &name, int force_size)
 {
     KIconLoader *loader = KIconLoader::global();
     return loader->loadIconSet(name, KIconLoader::Desktop, force_size);
 }
+#endif
 
+#if KICONTHEMES_BUILD_DEPRECATED_SINCE(5, 63)
 QPixmap BarIcon(const QString &name, int force_size, int state, const QStringList &overlays)
 {
     KIconLoader *loader = KIconLoader::global();
     return loader->loadIcon(name, KIconLoader::Toolbar, force_size, state, overlays);
 }
+#endif
 
-// deprecated
+#if KICONTHEMES_BUILD_DEPRECATED_SINCE(5, 0)
 QIcon BarIconSet(const QString &name, int force_size)
 {
     KIconLoader *loader = KIconLoader::global();
     return loader->loadIconSet(name, KIconLoader::Toolbar, force_size);
 }
+#endif
 
+#if KICONTHEMES_BUILD_DEPRECATED_SINCE(5, 63)
 QPixmap SmallIcon(const QString &name, int force_size, int state, const QStringList &overlays)
 {
     KIconLoader *loader = KIconLoader::global();
     return loader->loadIcon(name, KIconLoader::Small, force_size, state, overlays);
 }
+#endif
 
-// deprecated
+#if KICONTHEMES_BUILD_DEPRECATED_SINCE(5, 0)
 QIcon SmallIconSet(const QString &name, int force_size)
 {
     KIconLoader *loader = KIconLoader::global();
     return loader->loadIconSet(name, KIconLoader::Small, force_size);
 }
+#endif
 
+#if KICONTHEMES_BUILD_DEPRECATED_SINCE(5, 63)
 QPixmap MainBarIcon(const QString &name, int force_size, int state, const QStringList &overlays)
 {
     KIconLoader *loader = KIconLoader::global();
     return loader->loadIcon(name, KIconLoader::MainToolbar, force_size, state, overlays);
 }
+#endif
 
-// deprecated
+#if KICONTHEMES_BUILD_DEPRECATED_SINCE(5, 0)
 QIcon MainBarIconSet(const QString &name, int force_size)
 {
     KIconLoader *loader = KIconLoader::global();
@@ -1753,14 +1766,15 @@ QIcon MainBarIconSet(const QString &name, int force_size)
 }
 #endif
 
+#if KICONTHEMES_BUILD_DEPRECATED_SINCE(5, 65)
 QPixmap UserIcon(const QString &name, int state, const QStringList &overlays)
 {
     KIconLoader *loader = KIconLoader::global();
     return loader->loadIcon(name, KIconLoader::User, 0, state, overlays);
 }
+#endif
 
-// deprecated
-#ifndef KICONTHEMES_NO_DEPRECATED
+#if KICONTHEMES_BUILD_DEPRECATED_SINCE(5, 0)
 QIcon UserIconSet(const QString &name)
 {
     KIconLoader *loader = KIconLoader::global();
@@ -1768,11 +1782,13 @@ QIcon UserIconSet(const QString &name)
 }
 #endif
 
+#if KICONTHEMES_BUILD_DEPRECATED_SINCE(5, 66)
 int IconSize(KIconLoader::Group group)
 {
     KIconLoader *loader = KIconLoader::global();
     return loader->currentSize(group);
 }
+#endif
 
 QPixmap KIconLoader::unknown()
 {
@@ -1818,13 +1834,12 @@ void KIconLoader::setCustomPalette(const QPalette &palette)
 
 QPalette KIconLoader::customPalette() const
 {
-    return d->mPalette;
+    return d->mCustomPalette ? d->mPalette : QPalette();
 }
 
 void KIconLoader::resetPalette()
 {
     d->mCustomPalette = false;
-    d->mPalette = QPalette();
 }
 
 /*** the global icon loader ***/
@@ -1835,7 +1850,6 @@ KIconLoader *KIconLoader::global()
     return globalIconLoader();
 }
 
-#ifndef KICONTHEMES_NO_DEPRECATED
 void KIconLoader::newIconLoader()
 {
     if (global() == this) {
@@ -1845,7 +1859,6 @@ void KIconLoader::newIconLoader()
     reconfigure(objectName());
     emit iconLoaderSettingsChanged();
 }
-#endif
 
 void KIconLoader::emitChange(KIconLoader::Group g)
 {
