@@ -1,30 +1,18 @@
 /* vi: ts=8 sts=4 sw=4
- *
- * This file is part of the KDE project, module kdecore.
- * Copyright (C) 2000 Geert Jansen <jansen@kde.org>
- *                    Antonio Larrosa <larrosa@kde.org>
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License version 2 as published by the Free Software Foundation.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public License
- * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
- */
+
+    This file is part of the KDE project, module kdecore.
+    SPDX-FileCopyrightText: 2000 Geert Jansen <jansen@kde.org>
+    SPDX-FileCopyrightText: 2000 Antonio Larrosa <larrosa@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.0-only
+*/
 
 #ifndef KICONLOADER_H
 #define KICONLOADER_H
 
-#include <QtCore/QString>
-#include <QtCore/QStringList>
-#include <QtCore/QObject>
+#include <QString>
+#include <QStringList>
+#include <QObject>
 
 #include <kiconthemes_export.h>
 
@@ -39,6 +27,8 @@ class KIconTheme;
 class KPixmapSequence;
 
 /**
+ * @class KIconLoader kiconloader.h KIconLoader
+ *
  * Iconloader for KDE.
  *
  * KIconLoader will load the current icon theme and all its base themes.
@@ -76,12 +66,6 @@ class KPixmapSequence;
 class KICONTHEMES_EXPORT KIconLoader : public QObject
 {
     Q_OBJECT
-    Q_ENUMS(Context)
-    Q_ENUMS(Type)
-    Q_ENUMS(MatchType)
-    Q_ENUMS(Group)
-    Q_ENUMS(StdSizes)
-    Q_ENUMS(States)
 
 public:
     /**
@@ -92,7 +76,11 @@ public:
         Action,        ///< An action icon (e.g. 'save', 'print').
         Application,   ///< An icon that represents an application.
         Device,        ///< An icon that represents a device.
-        FileSystem,    ///< An icon that represents a file system. @deprecated Use Place instead.
+#if KICONTHEMES_ENABLE_DEPRECATED_SINCE(4, 8)
+        FileSystem,    ///< An icon that represents a file system. @deprecated Since 4.8. Use Place instead.
+#elif KICONTHEMES_BUILD_DEPRECATED_SINCE(4, 8)
+        FileSystem_DEPRECATED_DO_NOT_USE,
+#endif
         MimeType,      ///< An icon that represents a mime type (or file type).
         Animation,     ///< An icon that is animated.
         Category,      ///< An icon that represents a category.
@@ -102,6 +90,7 @@ public:
         Place,         ///< An icon that represents a location (e.g. 'home', 'trash').
         StatusIcon     ///< An icon that represents an event.
     };
+    Q_ENUM(Context)
 
     /**
      * The type of the icon.
@@ -111,6 +100,7 @@ public:
         Scalable, ///< Scalable-size icon.
         Threshold ///< A threshold icon.
     };
+    Q_ENUM(Type)
 
     /**
      * The type of a match.
@@ -119,6 +109,7 @@ public:
         MatchExact, ///< Only try to find an exact match.
         MatchBest   ///< Take the best match if there is no exact match.
     };
+    Q_ENUM(MatchType)
 
     /**
      * The group of the icon.
@@ -145,6 +136,7 @@ public:
         /// User icons
         User
     };
+    Q_ENUM(Group)
 
     /**
      * These are the standard sizes for icons.
@@ -163,6 +155,7 @@ public:
         /// enormous sized icons for iconviews
         SizeEnormous = 128
     };
+    Q_ENUM(StdSizes)
 
     /**
      * Defines the possible states of an icon.
@@ -171,8 +164,10 @@ public:
         DefaultState,  ///< The default state.
         ActiveState,   ///< Icon is active.
         DisabledState, ///< Icon is disabled.
+        SelectedState, ///< Icon is selected. @since 5.22
         LastState      ///< Last state (last constant)
     };
+    Q_ENUM(States)
 
     /**
      * Constructs an iconloader.
@@ -185,7 +180,7 @@ public:
      * KIconLoader::global(), so you hardly ever have to create an
      * iconloader object yourself. That one is the application's iconloader.
      */
-    explicit KIconLoader(const QString &appname = QString(), const QStringList &extraSearchPaths = QStringList(), QObject *parent = 0);
+    explicit KIconLoader(const QString &appname = QString(), const QStringList &extraSearchPaths = QStringList(), QObject *parent = nullptr);
 
     /**
      * Cleanup
@@ -238,13 +233,53 @@ public:
      */
     QPixmap loadIcon(const QString &name, KIconLoader::Group group, int size = 0,
                      int state = KIconLoader::DefaultState, const QStringList &overlays = QStringList(),
-                     QString *path_store = 0L,
+                     QString *path_store = nullptr,
+                     bool canReturnNull = false) const;
+
+    /**
+     * Loads an icon. It will try very hard to find an icon which is
+     * suitable. If no exact match is found, a close match is searched.
+     * If neither an exact nor a close match is found, a null pixmap or
+     * the "unknown" pixmap is returned, depending on the value of the
+     * @p canReturnNull parameter.
+     *
+     * @param name The name of the icon, without extension.
+     * @param group The icon group. This will specify the size of and effects to
+     * be applied to the icon.
+     * @param scale The scale of the icon group to use. If no icon exists in the
+     * scaled group, a 1x icon with its size multiplied by the scale will be
+     * loaded instead.
+     * @param size If nonzero, this overrides the size specified by @p group.
+     *             See KIconLoader::StdSizes.
+     * @param state The icon state: @p DefaultState, @p ActiveState or
+     * @p DisabledState. Depending on the user's preferences, the iconloader
+     * may apply a visual effect to hint about its state.
+     * @param overlays a list of emblem icons to overlay, by name
+     *                 @see drawOverlays
+     * @param path_store If not null, the path of the icon is stored here,
+     *        if the icon was found. If the icon was not found @p path_store
+     *        is unaltered even if the "unknown" pixmap was returned.
+     * @param canReturnNull Can return a null pixmap? If false, the
+     *        "unknown" pixmap is returned when no appropriate icon has been
+     *        found. <em>Note:</em> a null pixmap can still be returned in the
+     *        event of invalid parameters, such as empty names, negative sizes,
+     *        and etc.
+     * @return the QPixmap. Can be null when not found, depending on
+     *         @p canReturnNull.
+     * @since 5.48
+     */
+    // TODO KF6 merge loadIcon() and loadScaledIcon()
+    QPixmap loadScaledIcon(const QString &name, KIconLoader::Group group, qreal scale, int size = 0,
+                     int state = KIconLoader::DefaultState, const QStringList &overlays = QStringList(),
+                     QString *path_store = nullptr,
                      bool canReturnNull = false) const;
 
     /**
      * Loads an icon for a mimetype.
      * This is basically like loadIcon except that extra desktop themes are loaded if necessary.
      *
+     * Consider using QIcon::fromTheme() with a fallback to "application-octet-stream" instead.
+     *
      * @param iconName The name of the icon, without extension, usually from KMimeType.
      * @param group The icon group. This will specify the size of and effects to
      * be applied to the icon.
@@ -261,17 +296,18 @@ public:
      */
     QPixmap loadMimeTypeIcon(const QString &iconName, KIconLoader::Group group, int size = 0,
                              int state = KIconLoader::DefaultState, const QStringList &overlays = QStringList(),
-                             QString *path_store = 0) const;
+                             QString *path_store = nullptr) const;
 
     /**
      * Loads a pixmapSequence given the xdg icon name
      *
-     * @param name The name of the icon, without extension.
+     * @param iconName The name of the icon, without extension.
      * @param size the size/group to be used
      * @since 5.0
      */
     KPixmapSequence loadPixmapSequence(const QString &iconName, int size = SizeSmall) const;
 
+#if KICONTHEMES_ENABLE_DEPRECATED_SINCE(5, 0)
     /**
      * Creates an icon set, that will do on-demand loading of the icon.
      * Loading itself is done by calling loadIcon .
@@ -287,10 +323,10 @@ public:
      * @return the icon set. Can be null when not found, depending on
      *          @p canReturnNull.
      *
-     * @deprecated use QIcon::fromTheme instead, which uses the iconloader internally
+     * @deprecated Since 5.0, use QIcon::fromTheme instead, which uses the iconloader internally
      */
-#ifndef KICONTHEMES_NO_DEPRECATED
-    KICONTHEMES_DEPRECATED QIcon loadIconSet(const QString &name, KIconLoader::Group group, int size = 0,
+    KICONTHEMES_DEPRECATED_VERSION(5, 0, "Use QIcon::fromTheme(const QString&)")
+    QIcon loadIconSet(const QString &name, KIconLoader::Group group, int size = 0,
             bool canReturnNull = false);
 #endif
 
@@ -311,6 +347,26 @@ public:
     QString iconPath(const QString &name, int group_or_size,
                      bool canReturnNull = false) const;
 
+    /**
+     * Returns the path of an icon.
+     * @param name The name of the icon, without extension. If an absolute
+     * path is supplied for this parameter, iconPath will return it
+     * directly.
+     * @param group_or_size If positive, search icons whose size is
+     * specified by the icon group @p group_or_size. If negative, search
+     * icons whose size is - @p group_or_size.
+     *             See KIconLoader::Group and KIconLoader::StdSizes
+     * @param canReturnNull Can return a null string? If not, a path to the
+     *                      "unknown" icon will be returned.
+     * @param scale The scale of the icon group.
+     * @return the path of an icon, can be null or the "unknown" icon when
+     *         not found, depending on @p canReturnNull.
+     * @since 5.48
+     */
+    // TODO KF6 merge iconPath() with and without "scale" and move that argument after "group_or_size"
+    QString iconPath(const QString &name, int group_or_size,
+                     bool canReturnNull, qreal scale) const;
+
     /**
      * Loads an animated icon.
      * @param name The name of the icon.
@@ -321,7 +377,7 @@ public:
      * @return A QMovie object. Can be null if not found or not valid.
      *         Ownership is passed to the caller.
      */
-    QMovie *loadMovie(const QString &name, KIconLoader::Group group, int size = 0, QObject *parent = 0) const;
+    QMovie *loadMovie(const QString &name, KIconLoader::Group group, int size = 0, QObject *parent = nullptr) const;
 
     /**
      * Returns the path to an animated icon.
@@ -447,8 +503,8 @@ public:
      * Draws overlays on the specified pixmap, it takes the width and height
      * of the pixmap into consideration
      * @param overlays List of up to 4 overlays to blend over the pixmap. The first overlay
-     *                 will be in the bottom left corner, followed by bottom right, top right
-     *                 and top left. An empty QString can be used to leave the specific position
+     *                 will be in the bottom right corner, followed by bottom left, top left
+     *                 and top right. An empty QString can be used to leave the specific position
      *                 blank.
      * @param pixmap to draw on
      * @since 4.7
@@ -456,15 +512,46 @@ public:
     void drawOverlays(const QStringList &overlays, QPixmap &pixmap, KIconLoader::Group group, int state = KIconLoader::DefaultState) const;
 
     bool hasIcon(const QString &iconName) const;
+
+    /**
+     * Sets the colors for this KIconLoader.
+     * NOTE: if you set a custom palette, if you are using some colors from 
+     * application's palette, you need to track the application palette changes by yourself.
+     * If you no longer wish to use a custom palette, use resetPalette()
+     * @see resetPalette
+     * @since 5.39
+     */
+    void setCustomPalette(const QPalette &palette);
+
+    /**
+     * The colors that will be used for the svg stylesheet in case the
+     * loaded icons are svg-based, icons can be colored in different ways in
+     * different areas of the application
+     * @return the palette, if any, an invalid one if the user didn't set it
+     * @since 5.39
+     */
+    QPalette customPalette() const;
+
+    /**
+     * Resets the custom palette used by the KIconLoader to use the
+     * QGuiApplication::palette() again (and to follow it in case the
+     * application's palette changes)
+     * @since 5.39
+     */
+    void resetPalette();
+
 public Q_SLOTS:
+    // TODO: while marked as deprecated, newIconLoader() is still used:
+    // internally by KIconLoadeer as well as by Plasma's Icons kcm module (state: 5.17)
+    // this needs some further cleanup work before removing it from the API with KICONTHEMES_ENABLE_DEPRECATED_SINCE
     /**
      * Re-initialize the global icon loader
      *
-     * @deprecated since 5.0, use emitChange(Group)
+     * @todo Check deprecation, still used internally.
+     * @deprecated Since 5.0, use emitChange(Group)
      */
-#ifndef KICONTHEMES_NO_DEPRECATED
-    KICONTHEMES_DEPRECATED void newIconLoader();
-#endif
+    KICONTHEMES_DEPRECATED_VERSION(5, 0, "Use KIconLoader::emitChange(Group)")
+    void newIconLoader();
 
     /**
      * Emits an iconChanged() signal on all the KIconLoader instances in the system
@@ -497,90 +584,129 @@ private:
     Q_PRIVATE_SLOT(d, void _k_refreshIcons(int group))
 };
 
+#if KICONTHEMES_ENABLE_DEPRECATED_SINCE(5, 63)
 /**
  * \relates KIconLoader
  * Load a desktop icon.
+ * @deprecated since 5.63. Prefer QIcon::fromTheme instead where possible,
+ * if you need a pixmap use QIcon::pixmap with KIconLoader::StdSizes,
+ * if you need the overlay, use KIconLoader::loadIcon.
  */
+KICONTHEMES_DEPRECATED_VERSION(5, 63, "See API dox for replacement")
 KICONTHEMES_EXPORT QPixmap DesktopIcon(const QString &name, int size = 0,
                                        int state = KIconLoader::DefaultState, const QStringList &overlays = QStringList());
+#endif
 
+#if KICONTHEMES_ENABLE_DEPRECATED_SINCE(5, 0)
 /**
  * \relates KIconLoader
  * Load a desktop icon, and apply the necessary effects to get an IconSet.
- * @deprecated use QIcon::fromTheme instead
+ * @deprecated Since 5.0, use QIcon::fromTheme instead
  */
-#ifndef KICONTHEMES_NO_DEPRECATED
-KICONTHEMES_DEPRECATED_EXPORT QIcon DesktopIconSet(const QString &name, int size = 0);
+KICONTHEMES_DEPRECATED_VERSION(5, 0, "Use QIcon::fromTheme(const QString&)")
+KICONTHEMES_EXPORT QIcon DesktopIconSet(const QString &name, int size = 0);
 #endif
 
+#if KICONTHEMES_ENABLE_DEPRECATED_SINCE(5, 63)
 /**
  * \relates KIconLoader
  * Load a toolbar icon.
+ * @deprecated since 5.63. Prefer QIcon::fromTheme instead where possible,
+ * if you need a pixmap use QIcon::pixmap with KIconLoader::StdSizes,
+ * if you need the overlay, use KIconLoader::loadIcon.
  */
+KICONTHEMES_DEPRECATED_VERSION(5, 63, "See API dox for replacement")
 KICONTHEMES_EXPORT QPixmap BarIcon(const QString &name, int size = 0, int state = KIconLoader::DefaultState,
                                    const QStringList &overlays = QStringList());
+#endif
 
+#if KICONTHEMES_ENABLE_DEPRECATED_SINCE(5, 0)
 /**
  * \relates KIconLoader
  * Load a toolbar icon, and apply the necessary effects to get an IconSet.
- * @deprecated use QIcon::fromTheme instead
+ * @deprecated Since 5.0, use QIcon::fromTheme instead
  */
-#ifndef KICONTHEMES_NO_DEPRECATED
-KICONTHEMES_DEPRECATED_EXPORT QIcon BarIconSet(const QString &name, int size = 0);
+KICONTHEMES_DEPRECATED_VERSION(5, 0, "Use QIcon::fromTheme(const QString&)")
+KICONTHEMES_EXPORT QIcon BarIconSet(const QString &name, int size = 0);
 #endif
 
+#if KICONTHEMES_ENABLE_DEPRECATED_SINCE(5, 63)
 /**
  * \relates KIconLoader
  * Load a small icon.
+ * @deprecated since 5.63. Prefer QIcon::fromTheme instead where possible,
+ * if you need a pixmap use QIcon::pixmap with KIconLoader::StdSizes,
+ * if you need the overlay, use KIconLoader::loadIcon.
  */
+KICONTHEMES_DEPRECATED_VERSION(5, 63, "See API dox for replacement")
 KICONTHEMES_EXPORT QPixmap SmallIcon(const QString &name, int size = 0,
                                      int state = KIconLoader::DefaultState, const QStringList &overlays = QStringList());
+#endif
 
+#if KICONTHEMES_ENABLE_DEPRECATED_SINCE(5, 0)
 /**
  * \relates KIconLoader
  * Load a small icon, and apply the necessary effects to get an IconSet.
- * @deprecated use QIcon::fromTheme instead
+ * @deprecated Since 5.0, use QIcon::fromTheme instead
  */
-#ifndef KICONTHEMES_NO_DEPRECATED
-KICONTHEMES_DEPRECATED_EXPORT QIcon SmallIconSet(const QString &name, int size = 0);
+KICONTHEMES_DEPRECATED_VERSION(5, 0, "Use QIcon::fromTheme(const QString&)")
+KICONTHEMES_EXPORT QIcon SmallIconSet(const QString &name, int size = 0);
 #endif
 
+#if KICONTHEMES_ENABLE_DEPRECATED_SINCE(5, 63)
 /**
  * \relates KIconLoader
  * Load a main toolbar icon.
+ * @deprecated since 5.63. Prefer QIcon::fromTheme instead where possible,
+ * if you need a pixmap use QIcon::pixmap with KIconLoader::StdSizes,
+ * if you need the overlay, use KIconLoader::loadIcon.
  */
+KICONTHEMES_DEPRECATED_VERSION(5, 63, "See API dox for replacement")
 KICONTHEMES_EXPORT QPixmap MainBarIcon(const QString &name, int size = 0,
                                        int state = KIconLoader::DefaultState, const QStringList &overlays = QStringList());
+#endif
 
+#if KICONTHEMES_ENABLE_DEPRECATED_SINCE(5, 0)
 /**
  * \relates KIconLoader
  * Load a main toolbar icon, and apply the effects to get an IconSet.
- * @deprecated use QIcon::fromTheme instead
+ * @deprecated Since 5.0, use QIcon::fromTheme instead
  */
-#ifndef KICONTHEMES_NO_DEPRECATED
-KICONTHEMES_DEPRECATED_EXPORT QIcon MainBarIconSet(const QString &name, int size = 0);
+KICONTHEMES_DEPRECATED_VERSION(5, 0, "Use QIcon::fromTheme(const QString&)")
+KICONTHEMES_EXPORT QIcon MainBarIconSet(const QString &name, int size = 0);
 #endif
 
+#if KICONTHEMES_ENABLE_DEPRECATED_SINCE(5, 65)
 /**
  * \relates KIconLoader
  * Load a user icon. User icons are searched in $appdir/pics.
+ * @deprecated since 5.65 Prefer qrc files over user icon themes.
+ * If that's not an option for now, use KIconLoader::loadIcon.
  */
+KICONTHEMES_DEPRECATED_VERSION(5, 65, "See API dox for replacement")
 KICONTHEMES_EXPORT QPixmap UserIcon(const QString &name, int state = KIconLoader::DefaultState, const QStringList &overlays = QStringList());
+#endif
 
+#if KICONTHEMES_ENABLE_DEPRECATED_SINCE(5, 0)
 /**
  * \relates KIconLoader
  * Load a user icon, and apply the effects to get an IconSet.
- * @deprecated use QIcon::fromTheme instead
+ * @deprecated Since 5.0, use QIcon::fromTheme instead
  */
-#ifndef KICONTHEMES_NO_DEPRECATED
-KICONTHEMES_DEPRECATED_EXPORT QIcon UserIconSet(const QString &name);
+KICONTHEMES_DEPRECATED_VERSION(5, 0, "Use QIcon::fromTheme(const QString&)")
+KICONTHEMES_EXPORT QIcon UserIconSet(const QString &name);
 #endif
 
+#if KICONTHEMES_ENABLE_DEPRECATED_SINCE(5, 66)
 /**
  * \relates KIconLoader
  * Returns the current icon size for a specific group.
+ * @deprecated since 5.66 Prefer QStyle::pixelMetric.
+ * If that's not an option for now, use KIconLoader::currentSize.
  */
+KICONTHEMES_DEPRECATED_VERSION(5, 66, "Use QStyle::pixelMetric or KIconLoader::currentSize")
 KICONTHEMES_EXPORT int IconSize(KIconLoader::Group group);
+#endif
 
 namespace KDE
 {
@@ -592,14 +718,14 @@ namespace KDE
  * KDE style and effect standards.
  * @since 5.0
  */
-KICONTHEMES_EXPORT QIcon icon(const QString &iconName, KIconLoader *iconLoader = 0);
+KICONTHEMES_EXPORT QIcon icon(const QString &iconName, KIconLoader *iconLoader = nullptr);
 
 /**
  * \relates KIconLoader
  * Returns a QIcon for the given icon, with additional overlays.
  * @since 5.0
  */
-KICONTHEMES_EXPORT QIcon icon(const QString &iconName, const QStringList &overlays, KIconLoader *iconLoader = 0);
+KICONTHEMES_EXPORT QIcon icon(const QString &iconName, const QStringList &overlays, KIconLoader *iconLoader = nullptr);
 
 }
 
