@@ -1,44 +1,34 @@
 /* vi: ts=8 sts=4 sw=4
- *
- * kicontheme.cpp: Lowlevel icon theme handling.
- *
- * This file is part of the KDE project, module kdecore.
- * Copyright (C) 2000 Geert Jansen <jansen@kde.org>
- *                    Antonio Larrosa <larrosa@kde.org>
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License version 2 as published by the Free Software Foundation.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public License
- * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
- */
+
+    kicontheme.cpp: Lowlevel icon theme handling.
+
+    This file is part of the KDE project, module kdecore.
+    SPDX-FileCopyrightText: 2000 Geert Jansen <jansen@kde.org>
+    SPDX-FileCopyrightText: 2000 Antonio Larrosa <larrosa@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.0-only
+*/
 
 #include "kicontheme.h"
 
 #include <qplatformdefs.h>
+#include "debug.h"
 
 #include <QCoreApplication>
 #include <QAction>
-#include <QtCore/QMap>
-#include <QtCore/QSet>
-#include <QtCore/QFileInfo>
-#include <QtCore/QDir>
+#include <QMap>
+#include <QSet>
+#include <QFileInfo>
+#include <QDir>
 #include <QDebug>
 #include <QResource>
 
-#include <klocalizedstring.h> // KLocalizedString::localizedFilePath. Need such functionality in, hmm, QLocale? QStandardPaths?
+#include <KLocalizedString> // KLocalizedString::localizedFilePath. Need such functionality in, hmm, QLocale? QStandardPaths?
 
-#include <ksharedconfig.h>
+#include <KSharedConfig>
 
-#include <kconfiggroup.h>
+#include <KConfigGroup>
+#include <cmath>
 
 Q_GLOBAL_STATIC(QString, _themeOverride)
 
@@ -69,7 +59,18 @@ void initRCCIconTheme()
 }
 Q_COREAPP_STARTUP_FUNCTION(initRCCIconTheme)
 
+// Set the icon theme fallback to breeze
+// Most of our apps use "lots" of icons that most of the times
+// are only available with breeze, we still honour the user icon
+// theme but if the icon is not found there, we go to breeze
+// since it's almost sure it'll be there
+static void setBreezeFallback()
+{
+    QIcon::setFallbackThemeName(QStringLiteral("breeze"));
+}
 
+Q_COREAPP_STARTUP_FUNCTION(setBreezeFallback)
+class KIconThemeDir;
 class Q_DECL_HIDDEN KIconTheme::KIconThemePrivate
 {
 public:
@@ -85,7 +86,11 @@ public:
     QStringList mInherits;
     QStringList mExtensions;
     QVector<KIconThemeDir *> mDirs;
+    QVector<KIconThemeDir *> mScaledDirs;
     bool followsColorScheme : 1;
+
+    /// Searches the given dirs vector for a matching icon
+    QString iconPath(const QVector<KIconThemeDir *> &dirs, const QString &name, int size, qreal scale, KIconLoader::MatchType match) const;
 };
 Q_GLOBAL_STATIC(QString, _theme)
 Q_GLOBAL_STATIC(QStringList, _theme_list)
@@ -121,6 +126,10 @@ public:
     {
         return mSize;
     }
+    int scale() const
+    {
+        return mScale;
+    }
     int minSize() const
     {
         return mMinSize;
@@ -138,14 +147,100 @@ private:
     bool mbValid;
     KIconLoader::Type mType;
     KIconLoader::Context mContext;
-    int mSize, mMinSize, mMaxSize;
+    int mSize, mScale, mMinSize, mMaxSize;
     int mThreshold;
 
     const QString mBaseDir;
     const QString mThemeDir;
 };
 
+QString KIconTheme::KIconThemePrivate::iconPath(const QVector<KIconThemeDir *> &dirs, const QString &name, int size, qreal scale, KIconLoader::MatchType match) const
+{
+    QString path;
+    QString tempPath;      // used to cache icon path if it exists
 
+    int delta = -INT_MAX;  // current icon size delta of 'icon'
+    int dw = INT_MAX;      // icon size delta of current directory
+
+    // Rather downsample than upsample
+    int integerScale = std::ceil(scale);
+
+    // Search the directory that contains the icon which matches best to the requested
+    // size. If there is no directory which matches exactly to the requested size, the
+    // following criterias get applied:
+    // - Take a directory having icons with a minimum difference to the requested size.
+    // - Prefer directories that allow a downscaling even if the difference to
+    //   the requested size is bigger than a directory where an upscaling is required.
+    for (KIconThemeDir *dir : dirs) {
+        if (dir->scale() != integerScale) {
+            continue;
+        }
+
+        if (match == KIconLoader::MatchExact) {
+            if ((dir->type() == KIconLoader::Fixed) && (dir->size() != size)) {
+                continue;
+            }
+            if ((dir->type() == KIconLoader::Scalable) &&
+                    ((size < dir->minSize()) || (size > dir->maxSize()))) {
+                continue;
+            }
+            if ((dir->type() == KIconLoader::Threshold) &&
+                    (abs(dir->size() - size) > dir->threshold())) {
+                continue;
+            }
+        } else {
+            // dw < 0 means need to scale up to get an icon of the requested size.
+            // Upscaling should only be done if no larger icon is available.
+            if (dir->type() == KIconLoader::Fixed) {
+                dw = dir->size() - size;
+            } else if (dir->type() == KIconLoader::Scalable) {
+                if (size < dir->minSize()) {
+                    dw = dir->minSize() - size;
+                } else if (size > dir->maxSize()) {
+                    dw = dir->maxSize() - size;
+                } else {
+                    dw = 0;
+                }
+            } else if (dir->type() == KIconLoader::Threshold) {
+                if (size < dir->size() - dir->threshold()) {
+                    dw = dir->size() - dir->threshold() - size;
+                } else if (size > dir->size() + dir->threshold()) {
+                    dw = dir->size() + dir->threshold() - size;
+                } else {
+                    dw = 0;
+                }
+            }
+            // Usually if the delta (= 'dw') of the current directory is
+            // not smaller than the delta (= 'delta') of the currently best
+            // matching icon, this candidate can be skipped. But skipping
+            // the candidate may only be done, if this does not imply
+            // in an upscaling of the icon (it is OK to use a directory with
+            // smaller icons that what we've already found, however).
+            if ((abs(dw) >= abs(delta)) && ((dw < 0) || (delta > 0))) {
+                continue;
+            }
+        }
+
+        // cache the result of iconPath() call which checks if file exists
+        tempPath = dir->iconPath(name);
+
+        if (tempPath.isEmpty()) {
+            continue;
+        }
+
+        path = tempPath;
+
+        // if we got in MatchExact that far, we find no better
+        if (match == KIconLoader::MatchExact) {
+            return path;
+        }
+        delta = dw;
+        if (delta == 0) {
+            return path; // We won't find a better match anyway
+        }
+    }
+    return path;
+}
 
 KIconTheme::KIconTheme(const QString &name, const QString &appName, const QString &basePathHint)
     : d(new KIconThemePrivate)
@@ -207,7 +302,7 @@ KIconTheme::KIconTheme(const QString &name, const QString &appName, const QStrin
     }
 
     if (d->mDir.isEmpty()) {
-        qWarning() << "Icon theme" << name << "not found.";
+        qDebug() << "Icon theme" << name << "not found.";
         return;
     }
 
@@ -234,7 +329,8 @@ KIconTheme::KIconTheme(const QString &name, const QString &appName, const QStrin
     d->screenshot = cfg.readPathEntry("ScreenShot", QString());
     d->mExtensions = cfg.readEntry("KDE-Extensions", QStringList{ QStringLiteral(".png"), QStringLiteral(".svgz"), QStringLiteral(".svg"), QStringLiteral(".xpm") });
 
-    const QStringList dirs = cfg.readPathEntry("Directories", QStringList());
+    const QStringList dirs = cfg.readPathEntry("Directories", QStringList())
+                           + cfg.readPathEntry("ScaledDirectories", QStringList());
     for (QStringList::ConstIterator it = dirs.begin(); it != dirs.end(); ++it) {
         KConfigGroup cg(d->sharedConfig, *it);
         for (QStringList::ConstIterator itDir = themeDirs.constBegin(); itDir != themeDirs.constEnd(); ++itDir) {
@@ -243,7 +339,11 @@ KIconTheme::KIconTheme(const QString &name, const QString &appName, const QStrin
                 addedDirs.insert(currentDir);
                 KIconThemeDir *dir = new KIconThemeDir(*itDir, *it, cg);
                 if (dir->isValid()) {
-                    d->mDirs.append(dir);
+                    if (dir->scale() > 1) {
+                        d->mScaledDirs.append(dir);
+                    } else {
+                        d->mDirs.append(dir);
+                    }
                 } else {
                     delete dir;
                 }
@@ -270,6 +370,7 @@ KIconTheme::KIconTheme(const QString &name, const QString &appName, const QStrin
 KIconTheme::~KIconTheme()
 {
     qDeleteAll(d->mDirs);
+    qDeleteAll(d->mScaledDirs);
     delete d;
 }
 
@@ -310,7 +411,7 @@ QStringList KIconTheme::inherits() const
 
 bool KIconTheme::isValid() const
 {
-    return !d->mDirs.isEmpty();
+    return !d->mDirs.isEmpty() || !d->mScaledDirs.isEmpty();
 }
 
 bool KIconTheme::isHidden() const
@@ -345,7 +446,8 @@ QStringList KIconTheme::queryIcons(int size, KIconLoader::Context context) const
 {
     // Try to find exact match
     QStringList result;
-    foreach (KIconThemeDir* dir, d->mDirs) {
+    const auto listDirs = d->mDirs + d->mScaledDirs;
+    for (KIconThemeDir* dir : listDirs) {
         if ((context != KIconLoader::Any) && (context != dir->context())) {
             continue;
         }
@@ -401,8 +503,8 @@ QStringList KIconTheme::queryIconsByContext(int size, KIconLoader::Context conte
     // Usually, only the 0, 6 (22-16), 10 (32-22), 16 (48-32 or 32-16),
     // 26 (48-22) and 32 (48-16) will be used, but who knows if someone
     // will make icon themes with different icon sizes.
-
-    foreach (KIconThemeDir *dir, d->mDirs) {
+    const auto listDirs = d->mDirs + d->mScaledDirs;
+    for (KIconThemeDir* dir : listDirs) {
         if ((context != KIconLoader::Any) && (context != dir->context())) {
             continue;
         }
@@ -420,7 +522,8 @@ QStringList KIconTheme::queryIconsByContext(int size, KIconLoader::Context conte
 
 bool KIconTheme::hasContext(KIconLoader::Context context) const
 {
-    foreach (KIconThemeDir *dir, d->mDirs) {
+    const auto listDirs = d->mDirs + d->mScaledDirs;
+    for (KIconThemeDir* dir : listDirs) {
         if ((context == KIconLoader::Any) || (context == dir->context())) {
             return true;
         }
@@ -430,8 +533,13 @@ bool KIconTheme::hasContext(KIconLoader::Context context) const
 
 QString KIconTheme::iconPathByName(const QString &iconName, int size, KIconLoader::MatchType match) const
 {
-    foreach(const QString &current, d->mExtensions) {
-        const QString path = iconPath(iconName + current, size, match);
+    return iconPathByName(iconName, size, match, 1 /*scale*/);
+}
+
+QString KIconTheme::iconPathByName(const QString &iconName, int size, KIconLoader::MatchType match, qreal scale) const
+{
+    for (const QString &current : qAsConst(d->mExtensions)) {
+        const QString path = iconPath(iconName + current, size, match, scale);
         if (!path.isEmpty())
             return path;
     }
@@ -445,79 +553,17 @@ bool KIconTheme::followsColorScheme() const
 
 QString KIconTheme::iconPath(const QString &name, int size, KIconLoader::MatchType match) const
 {
-    QString path;
-    QString tempPath;      // used to cache icon path if it exists
-    int delta = -INT_MAX;  // current icon size delta of 'icon'
-    int dw = INT_MAX;      // icon size delta of current directory
+    return iconPath(name, size, match, 1 /*scale*/);
+}
 
-    // Search the directory that contains the icon which matches best to the requested
-    // size. If there is no directory which matches exactly to the requested size, the
-    // following criterias get applied:
-    // - Take a directory having icons with a minimum difference to the requested size.
-    // - Prefer directories that allow a downscaling even if the difference to
-    //   the requested size is bigger than a directory where an upscaling is required.
-    foreach (KIconThemeDir *dir, d->mDirs) {
-        if (match == KIconLoader::MatchExact) {
-            if ((dir->type() == KIconLoader::Fixed) && (dir->size() != size)) {
-                continue;
-            }
-            if ((dir->type() == KIconLoader::Scalable) &&
-                    ((size < dir->minSize()) || (size > dir->maxSize()))) {
-                continue;
-            }
-            if ((dir->type() == KIconLoader::Threshold) &&
-                    (abs(dir->size() - size) > dir->threshold())) {
-                continue;
-            }
-        } else {
-            // dw < 0 means need to scale up to get an icon of the requested size.
-            // Upscaling should only be done if no larger icon is available.
-            if (dir->type() == KIconLoader::Fixed) {
-                dw = dir->size() - size;
-            } else if (dir->type() == KIconLoader::Scalable) {
-                if (size < dir->minSize()) {
-                    dw = dir->minSize() - size;
-                } else if (size > dir->maxSize()) {
-                    dw = dir->maxSize() - size;
-                } else {
-                    dw = 0;
-                }
-            } else if (dir->type() == KIconLoader::Threshold) {
-                if (size < dir->size() - dir->threshold()) {
-                    dw = dir->size() - dir->threshold() - size;
-                } else if (size > dir->size() + dir->threshold()) {
-                    dw = dir->size() + dir->threshold() - size;
-                } else {
-                    dw = 0;
-                }
-            }
-            // Usually if the delta (= 'dw') of the current directory is
-            // not smaller than the delta (= 'delta') of the currently best
-            // matching icon, this candidate can be skipped. But skipping
-            // the candidate may only be done, if this does not imply
-            // in an upscaling of the icon (it is OK to use a directory with
-            // smaller icons that what we've already found, however).
-            if ((abs(dw) >= abs(delta)) && ((dw < 0) || (delta > 0))) {
-                continue;
-            }
-        }
+QString KIconTheme::iconPath(const QString &name, int size, KIconLoader::MatchType match, qreal scale) const
+{
+    // first look for a scaled image at exactly the requested size
+    QString path = d->iconPath(d->mScaledDirs, name, size, scale, KIconLoader::MatchExact);
 
-        // cache the result of iconPath() call which checks if file exists
-        tempPath = dir->iconPath(name);
-
-        if (tempPath.isEmpty()) {
-            continue;
-        }
-        path = tempPath;
-
-        // if we got in MatchExact that far, we find no better
-        if (match == KIconLoader::MatchExact) {
-            return path;
-        }
-        delta = dw;
-        if (delta == 0) {
-            return path; // We won't find a better match anyway
-        }
+    // then look for an unscaled one but request it at larger size so it doesn't become blurry
+    if (path.isEmpty()) {
+        path = d->iconPath(d->mDirs, name, size * scale, 1, match);
     }
     return path;
 }
@@ -545,7 +591,7 @@ QString KIconTheme::current()
     if (theme.isEmpty() || theme == QLatin1String("hicolor")) {
         // Still no theme, try config with kdeglobals.
         KConfigGroup cg(KSharedConfig::openConfig(), "Icons");
-        theme = cg.readEntry("Theme", QString());
+        theme = cg.readEntry("Theme", QStringLiteral("breeze"));
     }
     if (theme.isEmpty() || theme == QLatin1String("hicolor")) {
         // Still no good theme, use default.
@@ -581,7 +627,7 @@ QStringList KIconTheme::list()
     // These are not in the icon spec, but e.g. GNOME puts some icons there anyway.
     icnlibs += QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("pixmaps"), QStandardPaths::LocateDirectory);
 
-    Q_FOREACH (const QString &it, icnlibs) {
+    for (const QString &it : qAsConst(icnlibs)) {
         QDir dir(it);
         const QStringList lst = dir.entryList(QDir::Dirs | QDir::NoDotAndDotDot);
         for (QStringList::ConstIterator it2 = lst.begin(), total = lst.end(); it2 != total; ++it2) {
@@ -618,6 +664,7 @@ QString KIconTheme::defaultThemeName()
     return QStringLiteral("hicolor");
 }
 
+#if KICONTHEMES_BUILD_DEPRECATED_SINCE(5, 64)
 void KIconTheme::assignIconsToContextMenu(ContextMenus type,
         QList<QAction *> actions)
 {
@@ -651,6 +698,7 @@ void KIconTheme::assignIconsToContextMenu(ContextMenus type,
         break;
     }
 }
+#endif
 
 /*** KIconThemeDir ***/
 
@@ -658,6 +706,7 @@ KIconThemeDir::KIconThemeDir(const QString &basedir, const QString &themedir, co
     : mbValid(false)
     , mType(KIconLoader::Fixed)
     , mSize(config.readEntry("Size", 0))
+    , mScale(config.readEntry("Scale", 1))
     , mMinSize(1)    // just set the variables to something
     , mMaxSize(50)   // meaningful in case someone calls minSize or maxSize
     , mThreshold(2)
@@ -673,8 +722,10 @@ KIconThemeDir::KIconThemeDir(const QString &basedir, const QString &themedir, co
         mContext = KIconLoader::Device;
     } else if (tmp == QLatin1String("MimeTypes")) {
         mContext = KIconLoader::MimeType;
+#if KICONTHEMES_BUILD_DEPRECATED_SINCE(4, 8)
     } else if (tmp == QLatin1String("FileSystems")) {
         mContext = KIconLoader::FileSystem;
+#endif
     } else if (tmp == QLatin1String("Applications")) {
         mContext = KIconLoader::Application;
     } else if (tmp == QLatin1String("Actions")) {
@@ -695,10 +746,14 @@ KIconThemeDir::KIconThemeDir(const QString &basedir, const QString &themedir, co
         mContext = KIconLoader::StatusIcon;
     } else if (tmp == QLatin1String("Stock")) { // invalid, but often present context, skip warning
         return;
+    } else if (tmp == QLatin1String("Legacy")) { // invalid, but often present context for Adwaita, skip warning
+        return;
+    } else if (tmp == QLatin1String("UI")) { // invalid, but often present context for Adwaita, skip warning
+        return;
     } else if (tmp.isEmpty()) {
         // do nothing. key not required
     } else {
-        qWarning() << "Invalid Context=" << tmp << "line for icon theme: " << constructFileName(QString());
+        qCDebug(KICONTHEMES) << "Invalid Context=" << tmp << "line for icon theme: " << constructFileName(QString());
         return;
     }
     tmp = config.readEntry(QStringLiteral("Type"), QStringLiteral("Threshold"));
@@ -709,7 +764,7 @@ KIconThemeDir::KIconThemeDir(const QString &basedir, const QString &themedir, co
     } else if (tmp == QLatin1String("Threshold")) {
         mType = KIconLoader::Threshold;
     } else {
-        qWarning() << "Invalid Type=" << tmp << "line for icon theme: " << constructFileName(QString());
+        qCDebug(KICONTHEMES) << "Invalid Type=" << tmp << "line for icon theme: " << constructFileName(QString());
         return;
     }
     if (mType == KIconLoader::Scalable) {
